#!/usr/bin/env node
var program = require('commander');
var chalk = require('chalk');
var fs=require("fs");
var subject = require('./observerSubject');
var createDuckDir = require('./observers/createComponentDirectoryObserver');
var addActions = require('./observers/addActionsObserver');
var addReducer = require('./observers/addReducerObserver');
var addComponent = require('./observers/addComponentObserver');
var addStyles = require('./observers/addStylesObserver');
var addTests = require('./observers/addTestsObserver');
var addTypes = require('./observers/addTypesObserver');
var addState = require('./observers/addDefaultStateObserver');
var addAjax = require('./observers/addAjaxObserver');
var addDispatchers= require("./observers/addDispatchersObserver");

program
    .version('4.1.9')
    .arguments('<componentName>')
    .option('-d,--directory <directory>', 'The directory where your components will be housed.')
    .option('-e,--environment <environment>', 'This must be either web or native.')
    .option('-r,--redux <redux>', 'true/false. This specifies whether Redux integration should be done.')
    .action(function (componentName) {
        console.log(chalk.cyanBright('Please wait while we scaffold your component...'));
        //React Native has an issue if the component name doesn't start with a capital letter.
        componentName = !componentName?"":componentName.charAt(0).toUpperCase() + componentName.slice(1);
        let componentBaseFileName = componentName.split(/(?=[A-Z])/).join("-").toLowerCase();
        let directory = program.directory;
        let environment = program.environment;
        let addRedux = program.redux;

        try{            
            if(!directory)
            {
                throw new Error("Please set the -d or --directory flag.");
            }

            if(!environment)
            {
                throw new Error("Please set the -e or --environment flag.");
            }

            if (environment.toLowerCase() !== 'web' && environment.toLowerCase() !== 'native') {
                throw new Error('The -e or --environment flag must have a value of web or native.');
            }

            if (addRedux !== 'true' && addRedux !== 'false') {
                throw new Error('The -r or --redux flag must have a value of true or false.');
            }

            if(!componentName)
            {
                throw new Error("The component name must be specified.");
            }
            
            if(fs.existsSync(`${directory}/${componentBaseFileName}`))
            {
                throw new Error("There is already a component with the same name in the directory you supplied with the -d or --directory flag.");
            }


            console.log(chalk.bold.cyanBright(`Directory specified: ${directory}`));
            console.log(chalk.bold.cyanBright(`Environment specified: ${environment.toLowerCase()}`));
            console.log(chalk.bold.cyanBright(`Redux integration: ${addRedux}`));

            let SubjectInstance = new subject();

            SubjectInstance.Attach(new createDuckDir(directory, componentName, componentBaseFileName));
            SubjectInstance.Attach(new addTests(directory, componentName, componentBaseFileName, addRedux));
            SubjectInstance.Attach(new addComponent(directory, componentName, componentBaseFileName, addRedux));
            SubjectInstance.Attach(new addStyles(directory, componentName, componentBaseFileName, environment));
            SubjectInstance.Attach(new addAjax(directory, componentName, componentBaseFileName));
            

            if (addRedux === "true") {
                SubjectInstance.Attach(new addDispatchers(directory, componentName, componentBaseFileName));
                SubjectInstance.Attach(new addActions(directory, componentName, componentBaseFileName));
                SubjectInstance.Attach(new addReducer(directory, componentName, componentBaseFileName));
                SubjectInstance.Attach(new addTypes(directory, componentName, componentBaseFileName));
                SubjectInstance.Attach(new addState(directory, componentName, componentBaseFileName));
            }

            SubjectInstance.Notify();
        }
        catch(error)
        {
            console.log(chalk.bold.red((typeof error==="string")?error:error.message));
        }
    })
    .parse(process.argv);
